/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.scheduler;

import java.util.Date;
import java.util.Timer;
import org.openide.util.NbPreferences;

/**
 * Scheduler class to schedule updates from the web services
 * Will schedule a job every 12 hours
 * @author Merijn Zengers
 */
public final class WebServiceScheduler {
    //The delay between jobs 12 hours = 12 * 60 * 60 * 1000 = 43200000
    private static final int TIME_DELAY = 43200000;
    Timer timer = new Timer();
    private static final String LAST_UPDATED_PROPERTY_NAME = "lastUpdated"; 
    private static final long DEFAULT_LAST_UPDATED_VALUE = 0;
    
    /**
     * Initializes the scheduler.
     * Checks if the last update is done within the last 12 hours
     * will schedule an update with a delay of 12 hours minus time since last update
     * and a interval of 12 hours
     */
    public void init(){
        long lastUpdated = lastUpdated();
        long now = new Date().getTime();
        long timeSinceLastUpdate = now - lastUpdated;
        if(timeSinceLastUpdate > TIME_DELAY){
            schedule(0);
        } else {
            schedule(TIME_DELAY - timeSinceLastUpdate);
        }
    }
    
    /**
     * Returns when the web services where last updated
     * @return long the time since last update in milliseconds from epoch
     */
    private long lastUpdated(){
        return NbPreferences.forModule(this.getClass()).getLong(LAST_UPDATED_PROPERTY_NAME, DEFAULT_LAST_UPDATED_VALUE);
    }
    
    
    /**
     * Schedules a new execution.
     * Will set the lastUpdated property to now
     * @param the delay for first execution 
     */
    private void schedule(long delay){
        long time = new Date().getTime();
        NbPreferences.forModule(this.getClass()).putLong(LAST_UPDATED_PROPERTY_NAME, time);
        timer.schedule(new WebServiceUpdater(), delay, TIME_DELAY);
    }
            
    
}
